# Validation script for generation of NONMEMdata
# The validation will compare NONMEM and PSM objective function values

rm(list=ls())

library(PSM)
library(lattice)

REDO = FALSE

# This example demonstrates the simulation of a bolus dose
# in a 2-compartment system.
K12 <- 0.005
K21 <- 0.005
K10 <- 0.002  #[L/min]

Model.SimDose <- list(
                      Matrices = function(phi) {
                        V1i <- phi[["V1"]]
                        matA <- matrix(c(-(K12+K10) ,  K21 ,   
                                           K12 , -K21 ) ,nrow=2,byrow=T)
                        matC <- matrix(c(1/V1i,0),nrow=1)
                        list(matA=matA,matC=matC)
                      },
                      X0 = function(Time=Na,phi,U=Na) {
                        matrix(0,nrow=2)
                      },
                      SIG = function(phi) { 
                        sig22 <- phi[["sig22"]]
                        matrix(c( 0,0,
                                  0,sig22), nrow=2, byrow=T)
                      },
                      S = function(phi) {
                        matrix(phi[["S"]])
                      },
                      h = function(eta,theta,covar) {
                        phi <- theta
                        phi[["V1"]] <- phi[["V1"]]*exp(eta[1])
                        phi                      },
                      ModelPar = function(THETA){
                        list(theta=list(sig22=THETA[1], S=THETA[2],V1=THETA[3]),
                             OMEGA=matrix(THETA[4]) )
                      }
)

# Create Simulation Timeline 
SimDose.Subj <- 10
if(REDO) {
  SimDose.Data <- vector(mode="list",length=SimDose.Subj)
  for (i in 1:SimDose.Subj) {
    SimDose.Data[[i]]$Time <- seq(from=0,by=10,to=400)
    SimDose.Data[[i]]$Dose <- list(Time = c(30,180),State = c(1,1),Amount = c(1500,1500))
    }
}

#############
# Simulation
#############
#                               sig22  S   V1  OMEGA 
SimDose.THETA <- list( Init = c(10 ,  40 , 7,  .5 ))

if(REDO) {
  ModelCheck( Model.SimDose , SimDose.Data[[1]] , SimDose.THETA , DataHasY=FALSE)
  }

if(REDO) {
  SimDose.Data <- PSM.simulate(Model.SimDose, SimDose.Data, SimDose.THETA$Init, deltaTime=.1 )
  save(SimDose.Data, file="NONMEM.RData")
} else {
  load("NONMEM.RData")
}

# Extract etas
etas <- c()
for( i in seq_along(SimDose.Data)) {
  etas[i]  <- SimDose.Data[[i]]$eta
  }

####################################################################################
# Estimate using PSM

Est.Data <-   vector(mode="list",length=SimDose.Subj)
for( i in 1:length(SimDose.Data)) {
    Est.Data[[i]]$Time <- SimDose.Data[[i]]$Time
    Est.Data[[i]]$Y    <- SimDose.Data[[i]]$Y
    Est.Data[[i]]$Dose <- SimDose.Data[[i]]$Dose
  }

###############################################
### Comparison to NONMEM

### ODE Profiles
# Load ODE-Data
ODEFile <- "../ODE/TABLE.PAR"
ODEData <- read.table(  ODEFile , header=TRUE,skip=1)
# Load Pseudo ODE-Data
CSDEFile <- "../SDE1/TABLE.PAR"
CSDEData <- read.table(  CSDEFile , header=TRUE,skip=1)
CSDEData$TIME <- CSDEData$TIME -10   # Compensate for SDE init
CSDEData  <- subset( CSDEData , DV !=0.0)
# Plot illustrating the individual profiles from NM.ODE NM.SDE1 and PSM
NM.THETA <- c( 0 , 9.01^2 , 4.97,  .384 )
outODE <- APL.KF(NM.THETA, Model.SimDose, Est.Data ,longOutput=TRUE,Linear=TRUE)

# Plot the ODE profiles
par(mfrow=c(3,4))
yRange <- range( ODEData$IPRE , CSDEData$IPRE )
for( curID in 1:10) {
Par   <- Model.SimDose$ModelPar( NM.THETA  )
phi   <- Model.SimDose$h(eta=outODE$etaList[curID],theta=Par$theta,covar=NULL)
IDout <- LinKalmanFilter(phi, Model.SimDose, Est.Data[[curID]], echo = TRUE, outputInternals = TRUE, fast=TRUE)
NM.ODE  <- subset(  ODEData , ID ==curID & EVID==0)
NM.CSDE <- subset(  CSDEData , ID ==curID & EVID==0)
plot(  IDout$Time   ,IDout$Yp , type="l", main=paste("ID:",curID),ylim=yRange,xlab="TIME",ylab="Yp")
lines( NM.ODE$TIME  ,NM.ODE$IPRE, col="red"  , lty=2)   
lines( NM.CSDE$TIME ,NM.CSDE$IPRE, col="blue", lty=3) 
}
par(mfrow=c(1,1),new=TRUE) ; 
plot(0:1,0:1,type="n",axes=F,xlab="",ylab="") ; 
legend( .5 , .15 ,legend=c("PSM","NM.ODE","NM.CSDE"), col=c("black","red","blue") , lty=1:3)

# Difference plot
for( curID in 1:10) {
  Par   <- Model.SimDose$ModelPar( NM.THETA  )
  phi   <- Model.SimDose$h(eta=outODE$etaList[curID],theta=Par$theta,covar=NULL)
  IDout <- LinKalmanFilter(phi, Model.SimDose, Est.Data[[curID]], echo = TRUE, outputInternals = TRUE, fast=TRUE)
  NM.ODE  <- subset(  ODEData , ID==curID & EVID==0)
  NM.CSDE <- subset(  CSDEData, ID==curID & EVID==0)

  if(curID==1) {
    plot(  IDout$Time, IDout$Yp-NM.ODE$IPRE , type="l",ylim=c(-.1,.1))
    lines( IDout$Time, IDout$Yp-NM.CSDE$IPRE , type="l",col=2,lty=2)    

  } else {
    lines( IDout$Time, IDout$Yp-NM.ODE$IPRE , type="l")
    lines( IDout$Time, IDout$Yp-NM.CSDE$IPRE , type="l",col=2,lty=2)    
  }  
}

#################################################################
### SDE Profiles

# PSM  Optimization
#                                   sig22     S   V1    OMEGA 
THETA <- list( LB=rep(0,4),Init = c( 1 ,  9.01^2 , 4.97,  .5 ),UB=rep(100,4))
# Check Model and Data
ModelCheck(Model.SimDose, Est.Data[[1]], THETA,DataHasY=TRUE)
# Estimation with R
PSMout <- PSM.estimate(Model.SimDose, Est.Data, THETA,  trace = 1)

### SDE PROFILES
SDE2File <- "../SDE2/TABLE.PAR"
SDE2Data <- read.table(  SDE2File , header=TRUE,skip=1)
SDE2Data$TIME <- SDE2Data$TIME -10   # Compensate for SDE init

# Plot illustrating the individual profiles from NM.SDE2 and PSM
par(mfrow=c(3,4))
NM.THETA <- c(12.1, 6.11^2 , 4.91,  .369 )
outSDE<- APL.KF(NM.THETA, Model.SimDose, Est.Data ,longOutput=TRUE,Linear=TRUE)
yRange <- range( SDE2Data$IPRE )
for( curID in 1:10) {
Par   <- Model.SimDose$ModelPar( NM.THETA )
phi   <- Model.SimDose$h(eta=outSDE$etaList[curID],theta=Par$theta,covar=NULL)
IDout <- LinKalmanFilter(phi, Model.SimDose, Est.Data[[curID]], outputInternals = TRUE)
NM.SDE2 <- subset(  SDE2Data , ID==curID & EVID==0)
plot(  IDout$Time   ,IDout$Yp , type="l", main=paste("ID:",curID),ylim=yRange,xlab="TIME",ylab="Yp")
lines( NM.SDE2$TIME ,NM.SDE2$IPRE, col="blue", lty=3) 
}
par(mfrow=c(1,1),new=TRUE) ; plot(0:1,0:1,type="n",axes=F,xlab="",ylab="") ; legend( .5 , .15 ,legend=c("PSM","NM.SDE"), col=c("black","blue") , lty=c(1,3))


#### ETA COMPARISON
# ODE etas - Plot
NM.THETA <-  c( 0 , 9.01^2 , 4.97,  .384 )
outCSDE <- APL.KF(NM.THETA, Model.SimDose, Est.Data ,longOutput=TRUE,Linear=TRUE)
Tmp1    <-  unique( ODEData[,c("ID","ETA1")])
Tmp2    <-  unique( CSDEData[,c("ID","ETA1")])
yRange  <- max(abs(range( outCSDE$etaList ,Tmp1$ETA1,Tmp2$ETA1)))
plot(1:10 , outCSDE$etaList, type="p",xlab="ID",   col=2,  pch=1,ylim=c(-yRange,yRange),ylab="etas")
points( Tmp1$ID, Tmp1$ETA1, type="p",   col=3,  pch=2)
points( Tmp2$ID, Tmp2$ETA1, type="p",   col=4,  pch=8)
abline(h=0)
legend("topleft",legend=c("PSM","NM.ODE","NM.CSDE") , col=2:4 , pch=c(1,2,8))
title("ODE-Model: Estimated etas")


# SDE etas
NM.THETA <- c(12.1, 6.11^2 , 4.91,  .369 )
outSDE <- APL.KF(NM.THETA, Model.SimDose, Est.Data ,longOutput=TRUE,Linear=TRUE)
Tmp1    <-  unique( SDE2Data[,c("ID","ETA1")])
yRange  <- max(abs(range( etas, outSDE$etaList ,Tmp1$ETA1)))
plot(1:10 , outSDE$etaList, type="p",xlab="ID",   col=2,  pch=1,ylim=c(-yRange,yRange),ylab="etas")
points( Tmp1$ID, Tmp1$ETA1, type="p",   col=3,  pch=2)
abline(h=0)
legend("topleft",legend=c("PSM","NM.SDE") , col=2:3 , pch=c(1,2))
title("SDE-Model: Estimated etas")


# ETA CHECK with identical Population Parameters
# NONMEM run: SDE3
SDE3File <- "../SDE3/TABLE.PAR"
SDE3Data <- read.table(  SDE3File , header=TRUE,skip=1)
SDE3Data$TIME <- SDE3Data$TIME -10   # Compensate for SDE init
SDE3Data   <-  unique( SDE3Data[,c("ID","ETA1")])
SDE3Data$ID <- as.numeric(SDE3Data$ID)
SDE3Data    <- SDE3Data[ order(SDE3Data$ID) ,]
NM.THETA <- c(12.1, 6.11^2 , 4.91,  .369 )
outSDE <- APL.KF(NM.THETA, Model.SimDose, Est.Data ,longOutput=TRUE)
yRange  <- max(abs(range( outSDE$etaList ,SDE3Data$ETA1)))

plot(1:10 , outSDE$etaList, type="p",xlab="ID", ylab="ETA",  col=2,  pch=1,ylim=c(-yRange,yRange))
points(SDE3Data$ID, SDE3Data$ETA1, type="p", col=3, pch=2 )
abline(h=0)
title("SDE Eta Comparison with IDENTICAL POP.Par")
legend("topleft",legend=c("PSM","NM.SDE") , col=c(2,3) , pch=c(1,2))



# ETAs simulated vs. estimated
NM.THETA <- c(10 ,  40 , 7,  .5 )
outEST <- APL.KF(NM.THETA, Model.SimDose, Est.Data ,longOutput=TRUE,Linear=TRUE)
yRange  <- max(abs(range( outEST$etaList ,etas)))
plot(1:10 , outEST$etaList, type="p",xlab="ID", ylab="ETA",  col=3,  pch=3,ylim=c(-yRange,yRange))
points(1:10, etas , type="p", col=2 )
abline(h=0)
title("Simulated etas vs. PSM estimated with same fixed parameters")


#################################################################
##### Likelihood comparisson
(N <- nrow( subset(ODEData, EVID==0)))

# NONMEM LL
# ODE OBF: 2306.517
#Check LL OFV from NONMEM ODE estimated parameters
#            sig22  S        V1    OMEGA
NM.THETA <- c( 0 , 9.01^2 , 4.97,  .384 )
outODE <- APL.KF(NM.THETA, Model.SimDose, Est.Data ,longOutput=TRUE,Linear=TRUE)
outODE$negLogLike
2*outODE$negLogLike - log(2*pi)*N
2*outODE$negLogLike - log(2*pi)*N - 2306.517

# NONMEM LL
# SDE OBF: 2070.289
#NONMEM estimated parameters
#            sig22  S        V1    OMEGA
NM.THETA <- c(12.1, 6.11^2 , 4.91,  .369 )
outSDE<- APL.KF(NM.THETA, Model.SimDose, Est.Data ,longOutput=TRUE,Linear=TRUE)
outSDE$negLogLike
2*outSDE$negLogLike - log(2*pi)*N
2*outSDE$negLogLike - log(2*pi)*N -  2070.289
